﻿/**
 * @fileoverview 特定記号に囲まれた数字を丸数字、黒丸数字などに置き換える
 * @author Kenshi Muto &lt;kmuto@debian.org&gt;
 * @requires libCommon.jsx
*/

/*
    Copyright:
    2012 Kenshi Muto <kmuto@debian.org>
----------------------------------------------------------------------
ソフトウェア使用許諾同意書
本ソフトウェアの利用・変更・再配布にあたっては、下記の使用許諾同意書に
同意する必要があります。

1. 本使用許諾同意書における「ソフトウェア」とは、機械可読の資料 (ライブ
   ラリ、スクリプト、ソースファイル、データファイル)、実行形式、および
   文書を意味します。
2. 本ソフトウェアの使用許諾同意書に同意する限りにおいて、使用者は
   本ソフトウェアを自由に利用、変更することができます。
3. 本ソフトウェアに変更を加えない限りにおいて、使用者は本ソフトウェアを
   自由にコピー、再配布することができます。
4. 本ソフトウェアは無保証です。作者およびそれに関連する組織、配布者は、
   本ソフトウェアの使用に起因する一切の直接損害、間接損害、偶発的損害、
   特別損害、懲戒的損害、派生的損害について何らの責任・保証も負いません。
5. 本ソフトウェアを変更した上で再配布するときには、下記の事項すべてに
   従わなければなりません。
   - 使用許諾同意書の内容に変更を加えてはなりません。技術上の理由で
     文字エンコーディングの変換を行うことは許可しますが、その使用者が
     特殊な技術的措置なしに可読な形でなければなりません。
   - 技術上の理由でバイナリ化・難読化を行う場合も、変更箇所を含めた
     ソフトウェアを、その使用者が可読可能な形式の形で同一のメディアで
     提供しなければなりません。本使用許諾同意書の2条および3条により、
     使用者が可読形式の該当ソフトウェアを変更、コピー、再配布することを
     妨げてはなりません。
   - ソフトウェア構成物の所定の作者名の欄に、変更者のクレジット
     (個人名、企業名、所属、連絡先など)を「追加」しなければなりません。
6. 本ソフトウェアを変更した上で再配布するときには、変更理由および
   その内容を明記することが推奨されます。
7. 使用者がソフトウェアに適用可能な特許に対して特許侵害にかかわる何らか
   の行動を開始した時点で、この使用許諾同意書は自動的に終了し、以降
   使用者はこの使用許諾書によって与えられた一切の権利を放棄するものと
   します。

著作権所有者 Copyright (C) 2012 Kenshi Muto. All rights reserved.
使用許諾同意書バージョン1.0
著作権所有者による書面での事前の許可がない限り、この使用許諾同意書
に変更を加えてはなりません。

CID変換テーブルは次のスクリプトから流用しています。

「round_num.jsx」 (c) 2008-2009 www.seuzo.jp
http://d.hatena.ne.jp/seuzo/20090527/1243353039
GNU GPLv3 (http://sourceforge.jp/projects/opensource/wiki/licenses%252FGNU_General_Public_License_version_3.0)
----------------------------------------------------------------------
*/
#include "../libs/libCommon.jsx"

/**
 * ドキュメント中の、指定文字とそれに囲まれた数値を、白丸数字・黒丸数字・白角囲み数字・白角丸囲み数字・黒角囲み数字・黒角丸囲み数字・括弧囲み数字に変換する。変換できる数値の上限は種類および使用フォントに依存する。なお、合成フォントはInDesignの仕様で変換できない。
 * @param {Object} obj パラメータオブジェクト
 * @param {Document} obj.document ドキュメントオブジェクト
 * @param {Font} obj.findfont 対象フォントオブジェクト
 * @param {String} obj.prefix 数値を囲む前修飾子(たとえば<)。正規表現記号に該当する文字列の場合はエスケープ済みのものを指定すること
 * @param {String} obj.postfix 数値を囲む後修飾子(たとえば>)。正規表現記号に該当する文字列の場合はエスケープ済みのものを指定すること
 * @param {String} obj.pattern 数値変換の種類。maru(白丸数字)・kuromaru(黒丸数字)・kaku(白角囲み数字)・kadomaru(白角丸囲み数字)・kurokaku(黒角囲み数字)・kurokadomaru(黒角丸囲み数字)・kakko(括弧囲み数字)
 * @type Nothing
*/
function roundNumbers(obj) {
  app.findGrepPreferences = NothingEnum.NOTHING;
  app.findChangeGrepOptions = NothingEnum.NOTHING;
  app.findChangeGrepOptions.includeFootnotes = true;
  app.findChangeGrepOptions.includeMasterPages = false;
  app.findChangeGrepOptions.widthSensitive = true;
  app.findGrepPreferences.findWhat = obj.prefix + "\\d+" + obj.postfix;
  if (obj.findfont != null) app.findGrepPreferences.appliedFont = obj.findfont;

  var matches = obj.document.findGrep(true);

  for (var i = 0; i < matches.length; i++) {
     var num;
    try {
      num = matches[i].contents.replace(obj.prefix, "").replace(obj.postfix, "");
	} catch(e) {
      num = matches[i].texts[0].contents.replace(obj.prefix, "").replace(obj.postfix, "");
    }
    var ch = getCID(obj.pattern, num);
    if (ch == null) {
      alert(num + " の字体はありません。飛ばします。");
      continue;
    }

    matches[i].select();
    var font = matches[i].appliedFont;
    if (getCSVersion() > 3) {
      matches[i].contents = "λ";
    } else {
      matches[i].insertionPoints[-1].contents = "λ";
      matches[i].remove();
    }
    var target = obj.document.selection[0];

    app.findGlyphPreferences = NothingEnum.NOTHING;
    app.findChangeGlyphOptions = NothingEnum.NOTHING;
    app.changeGlyphPreferences = NothingEnum.NOTHING;
    app.findGlyphPreferences.appliedFont = font;
    app.changeGlyphPreferences.appliedFont = font;
    app.findGlyphPreferences.glyphID = 1045; // λ
    app.changeGlyphPreferences.glyphID = ch;
    try {
      var result = target.changeGlyph();
      result[0].clearOverrides(); // 上付きなどが標準にオーバライドされてしまうのをクリア
    } catch (e) {
      obj.document.undo(); // removeのundo
      obj.document.undo(); // insertionのundo
      alert(num + " の処理に失敗しました。おそらくフォントに字体がありません。");
    }
  }
}

/**
 * 変換種類と数値を指定し、該当のCID番号を返す
 * @param {String} pattern 数値変換の種類。maru(白丸数字)・kuromaru(黒丸数字)・kaku(白角囲み数字)・kadomaru(白角丸囲み数字)・kurokaku(黒角囲み数字)・kurokadomaru(黒角丸囲み数字)・kakko(括弧囲み数字)
 * @param {Number} num 変換する数値
 * @type Nummber
 * @return 文字CID番号。存在しない場合はnull
*/
function getCID(pattern, num) {
  var array;
  switch(pattern) {
    case "kuromaru":
    case "marukuro":
      return [10503, 8286, 8287, 8288, 8289, 8290, 8291, 8292, 8293, 8294, 10514, 10515, 10516, 10517, 10518, 10519, 10520, 10521, 10522, 10523, 10524, 10525, 10526, 10527, 10528, 10529, 10530, 10531, 10532, 10533, 10534, 10535, 10536, 10537, 10538, 10539, 10540, 10541, 10542, 10543, 10544, 10545, 10546, 10547, 10548, 10549, 10550, 10551, 10552, 10553, 10554, 10555, 10556, 10557, 10558, 10559, 10560, 10561, 10562, 10563, 10564, 10565, 10566, 10567, 10568, 10569, 10570, 10571, 10572, 10573, 10574, 10575, 10576, 10577, 10578, 10579, 10580, 10581, 10582, 10583, 10584, 10585, 10586, 10587, 10588, 10589, 10590, 10591, 10592, 10593, 10594, 10595, 10596, 10597, 10598, 10599, 10600, 10601, 10602, 10603, 10604, 10504, 10505, 10506, 10507, 10508, 10509, 10510, 10511, 10512, 10513][num];
    case "kaku":
    case "kado":
      return [10764, 10766, 10768, 10770, 10772, 10774, 10776, 10778, 10780, 10782, 10784, 10785, 10786, 10787, 10788, 10789, 10790, 10791, 10792, 10793, 10794, 10795, 10796, 10797, 10798, 10799, 10800, 10801, 10802, 10803, 10804, 10805, 10806, 10807, 10808, 10809, 10810, 10811, 10812, 10813, 10814, 10815, 10816, 10817, 10818, 10819, 10820, 10821, 10822, 10823, 10824, 10825, 10826, 10827, 10828, 10829, 10830, 10831, 10832, 10833, 10834, 10835, 10836, 10837, 10838, 10839, 10840, 10841, 10842, 10843, 10844, 10845, 10846, 10847, 10848, 10849, 10850, 10851, 10852, 10853, 10854, 10855, 10856, 10857, 10858, 10859, 10860, 10861, 10862, 10863, 10864, 10865, 10866, 10867, 10868, 10869, 10870, 10871, 10872, 10873, 10874, 10765, 10767, 10769, 10771, 10773, 10775, 10777, 10779, 10781, 10783][num];
    case "kadomaru":
    case "kakumaru":
    case "marukado":
      return [11307, 11309, 11311, 11313, 11315, 11317, 11319, 11321, 11323, 11325, 11327, 11328, 11329, 11330, 11331, 11332, 11333, 11334, 11335, 11336, 11337, 11338, 11339, 11340, 11341, 11342, 11343, 11344, 11345, 11346, 11347, 11348, 11349, 11350, 11351, 11352, 11353, 11354, 11355, 11356, 11357, 11358, 11359, 11360, 11361, 11362, 11363, 11364, 11365, 11366, 11367, 11368, 11369, 11370, 11371, 11372, 11373, 11374, 11375, 11376, 11377, 11378, 11379, 11380, 11381, 11382, 11383, 11384, 11385, 11386, 11387, 11388, 11389, 11390, 11391, 11392, 11393, 11394, 11395, 11396, 11397, 11398, 11399, 11400, 11401, 11402, 11403, 11404, 11405, 11406, 11407, 11408, 11409, 11410, 11411, 11412, 11413, 11414, 11415, 11416, 11417, 11308, 11310, 11312, 11314, 11316, 11318, 11320, 11322, 11324, 11326][num];
    case "kurokaku":
    case "kurokado":
    case "kakukuro":
      return [11037, 11039, 11041, 11043, 11045, 11047, 11049, 11051, 11053, 11055, 11057, 11058, 11059, 11060, 11061, 11062, 11063, 11064, 11065, 11066, 11067, 11068, 11069, 11070, 11071, 11072, 11073, 11074, 11075, 11076, 11077, 11078, 11079, 11080, 11081, 11082, 11083, 11084, 11085, 11086, 11087, 11088, 11089, 11090, 11091, 11092, 11093, 11094, 11095, 11096, 11097, 11098, 11099, 11100, 11101, 11102, 11103, 11104, 11105, 11106, 11107, 11108, 11109, 11110, 11111, 11112, 11113, 11114, 11115, 11116, 11117, 11118, 11119, 11120, 11121, 11122, 11123, 11124, 11125, 11126, 11127, 11128, 11129, 11130, 11131, 11132, 11133, 11134, 11135, 11136, 11137, 11138, 11139, 11140, 11141, 11142, 11143, 11144, 11145, 11146, 11147, 11038, 11040, 11042, 11044, 11046, 11048, 11050, 11052, 11054, 11056][num];
    case "kurokadomaru":
    case "kurokakumaru":
    case "kadokuromaru":
      return [11576, 11578, 11580, 11582, 11584, 11586, 11588, 11590, 11592, 11594, 11596, 11597, 11598, 11599, 11600, 11601, 11602, 11603, 11604, 11605, 11606, 11607, 11608, 11609, 11610, 11611, 11612, 11613, 11614, 11615, 11616, 11617, 11618, 11619, 11620, 11621, 11622, 11623, 11624, 11625, 11626, 11627, 11628, 11629, 11630, 11631, 11632, 11633, 11634, 11635, 11636, 11637, 11638, 11639, 11640, 11641, 11642, 11643, 11644, 11645, 11646, 11647, 11648, 11649, 11650, 11651, 11652, 11653, 11654, 11655, 11656, 11657, 11658, 11659, 11660, 11661, 11662, 11663, 11664, 11665, 11666, 11667, 11668, 11669, 11670, 11671, 11672, 11673, 11674, 11675, 11676, 11677, 11678, 11679, 11680, 11681, 11682, 11683, 11684, 11685, 11686, 11577, 11579, 11581, 11583, 11585, 11587, 11589, 11591, 11593, 11595][num];
    case "kakko":
      return [8227, 8071, 8072, 8073, 8074, 8075, 8076, 8077, 8078, 8079, 8080, 8081, 8082, 8083, 8084, 8085, 8086, 8087, 8088, 8089, 8090, 9894, 9895, 9896, 9897, 9898, 9899, 9900, 9901, 9902, 9903, 9904, 9905, 9906, 9907, 9908, 9909, 9910, 9911, 9912, 9913, 9914, 9915, 9916, 9917, 9918, 9919, 9920, 9921, 9922, 9923, 9924, 9925, 9926, 9927, 9928, 9929, 9930, 9931, 9932, 9933, 9934, 9935, 9936, 9937, 9938, 9939, 9940, 9941, 9942, 9943, 9944, 9945, 9946, 9947, 9948, 9949, 9950, 9951, 9952, 9953, 9954, 9955, 9956, 9957, 9958, 9959, 9960, 9961, 9962, 9963, 9964, 9965, 9966, 9967, 9968, 9969, 9970, 9971, 9972, 9973, 9884, 9885, 9886, 9887, 9888, 9889, 9890, 9891, 9892, 9893][num];
    case "maru":
    default:
      return [8224, 7555, 7556, 7557, 7558, 7559, 7560, 7561, 7562, 7563, 7564, 7565, 7566, 7567, 7568, 7569, 7570, 7571, 7572, 7573, 7574, 8091, 8102, 8103, 8104, 8105, 8106, 8107, 8108, 8109, 8110, 8111, 10244, 10245, 10246, 10247, 10248, 10249, 10250, 10251, 10252, 10253, 10254, 10255, 10256, 10257, 10258, 10259, 10260, 10261, 10262, 10263, 10264, 10265, 10266, 10267, 10268, 10269, 10270, 10271, 10272, 10273, 10274, 10275, 10276, 10277, 10278, 10279, 10280, 10281, 10282, 10283, 10284, 10285, 10286, 10287, 10288, 10289, 10290, 10291, 10292, 10293, 10294, 10295, 10296, 10297, 10298, 10299, 10300, 10301, 10302, 10303, 10304, 10305, 10306, 10307, 10308, 10309, 10310, 10311, 10312, 10234, 10235, 10236, 10237, 10238, 10239, 10240, 10241, 10242, 10243][num];
  }
}
